# ReFuzz: A structure aware feedback-driven fuzzer for the Windows ReFS drive. Based on kAFL

## Credits:

ReFuzz is based on the incredible kAFL fuzzer: https://github.com/RUB-SysSec/kAFL

### Currently missing: 

- full documentation

## Setup

This is a short introduction on how to setup ReFuzz to fuzz the Microsoft ReFS kernel driver.
Please install the fuzzer on Ubuntu (16.04).

### Download ReFuzz and install patched QEMU and KVM
```
$ chmod u+x install.sh
$ sudo ./install.sh
$ sudo reboot
```

### Downlaod ReFS Mutator and install
```
cd /path/to/mutator
pip install .
```

### Prepare ReFS Image
* Create a ReFS formatted VHD on a Windows machine
* Use 'Metadata Extractor' to extract all metadata and other information
* Optionally you can use the provided data in data.zip to start the first fuzzing run

### Setup VM
* Create Windows VM with favored hypervisor and export as qcow2 image:
  * Retrieve an ISO file of the desired Windows OS and install it
  * Download ReFuzz and compile the loader agent:
  ```
  cd path/to/refuzz/kAFL-Fuzzer/agents/windows_x86_64
  chmod u+x compile.sh
  ./compile.sh
  ```
  * place loader agent in the Windows VM
  * place required ReFS artifacts in the Windows VM and note down paths to artifacts
    * full VHD image and translation table
* Shutdown the VM
     
### Prepare VM for kAFL fuzzing

* On the host: Create Overlay and Snapshot Files:

```
mkdir snapshot && cd snapshot
qemu-img create -b /absolute/path/to/hdd/windows.qcow2 -f qcow2 overlay_0.qcow2
qemu-img create -f qcow2 ram.qcow2 2000 
```

* Start the VM using QEMU-PT:

```
cd /path/to/refuzz
./qemu-2.9.0/x86_64-softmmu/qemu-system-x86_64 -hdb /path/to/snapshot/ram.qcow2 -hda /path/to/snapshot/overlay_0.qcow2 -machine pc-i440fx-2.6 -serial mon:stdio -enable-kvm -k de -m 2000 -vnc :1,password
# press CTRL-a + c
change vnc password
```

* Create a copy of the full ReFS VHD image and mount it once to force Windows to load the driver
* Delete the copy afterward
* Execute loader binary stored before on the Windows VM. VM should freeze. Switch to the QEMU management console and create a snapshot:

```
savevm kafl
q 
```

### Compile and configure kAFL components
* Edit `/path/to/refuzz/kAFL-Fuzzer/kafl.ini` (`qemu-kafl_location` to point to `path/to/refuzz/qemu-2.9.0/x86_64-softmmu/qemu-system-x86_64`)

* Compile agent:

```
cd <REFUZZ_ROOT>/kAFL-Fuzzer/agents/windows_x86_64
chmod u+x compile.sh
./compile.sh
```

* Retrieve address ranges of loaded drivers and note down range of refs.sys:

```
cd /path/to/refuzz/kAFL-Fuzzer
python kafl_info.py /path/to/snapshot/ram.qcow2 /path/to/snapshot/ agents/windows_x86_64/info/info 2000 -v
```

### Start Fuzzing!

* copy example_run.sh to my_run.sh and adjust values inside
* run script to start the fuzzer:

```
./my_run.sh
```
