#include <stdio.h>
#include <limits.h>

#include "expander.h"

typedef struct {
    unsigned long image_offset;
    unsigned long payload_offset;
    unsigned long size;
} table_element;


static table_element translation_table[100];

static int add_ttable_line_to_struct(char* line, int where){
    unsigned long ioffset;
    unsigned long poffset;
    unsigned long size;
    char pageType[10];

    sscanf(line, "%lx - %lx - %lx - %s", &ioffset, &poffset, &size, pageType);
    translation_table[where].image_offset = ioffset;
    translation_table[where].payload_offset = poffset;
    translation_table[where].size = size;
    if(ioffset > LONG_MAX){
        printQemu("[-] Expander: possible overflow");
        return 1;
    }
    return 0;
}

int init_translation_table(char* path_to_translation){
    FILE* stream;
    char line[64];
    int counter = 0;

    stream = fopen(path_to_translation, "r");
    if(stream == NULL) {
        printQemu("[-] Expander: cannot open translation table file");
        return 1;
    }

    while(fgets(line, 64, stream)){
        if(add_ttable_line_to_struct(line, counter)){
            fclose(stream);
            return 1;
        }
        counter++;
    }
    if(ferror(stream)){
        printQemu("[-] Expander: error while reading ttable file");
    }
    fclose(stream);

    translation_table[counter].size = 0;
    return 0;
}

int write_payload_to_file(kAFL_payload2* payload, char* path_to_image){
    FILE* image_stream;

    table_element* current;
    int payload_size = payload->size;
    void* data;
    int numwritten;

    image_stream = fopen(path_to_image, "r+b");
    if(image_stream == NULL){
        printQemu("[-] Expander: cannot open refs image");
        return 1;
    }

    for(current = translation_table;current->size !=0;current++){
        fseek(image_stream, current->image_offset, SEEK_SET);
        if(current->payload_offset + current->size > payload_size){
            printQemu("[-] Expander: payload data to small");
            fclose(image_stream);
            return 1;
        }
        data = &(payload->data[current->payload_offset]);
        numwritten = fwrite(data, sizeof(uint8_t), current->size, image_stream);
        if( numwritten != current->size ){
            printQemu("[-] Expander: failure in writing refs image");
            fclose(image_stream);
            return 1;
        }
    }

    fclose(image_stream);
    return 0;
}
