#include "refs_prpc.h"

#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* File: refs_prpc.c
 * Implements functions that are the same for caller and callee
 */

int callIsListEnd(ReFSCall* call)
{
	if (!call)
		return 1;
	if (call->id == LISTENDCALL)
		return 1;
	return 0;
}
//
int paramIsListEnd(ReFSCallParam* param)
{
	if (!param)
		return 1;
	// void parameters make no sense, so use empty parameters as list end indicator
	if (param->memsize == 0)
		return 1;
	return 0;
}

//
ReFSCall* getNextCall(ReFSCall* call)
{
	ReFSCall* next = (ReFSCall*)(((uint8_t*)call->params)+call->params_size);
	if (callIsListEnd(next))
		return NULL;
	return next;
}


ReFSCallParam* getFirstParam(ReFSCall* call)
{
	if (!call)
		return NULL;
	if (call->params_size == 0)
		return NULL;
	return call->params;
}
//
ReFSCallParam* getNextParam(ReFSCallParam* param)
{
	ReFSCallParam* next = (ReFSCallParam*)(param->mem+param->memsize);
	if (paramIsListEnd(next))
		return NULL;
	return next;
}

////////////////////////////////////////////////////////////////////////////////

void dbgmsg(const char* msg) {
	char* format = "[D] KAFL ReFS: %s\n";
	printf(format, msg);
}
//
void errmsg(const char* msg) {
	char* format = "[-] KAFL ReFS: %s\n";
	fprintf(stderr, format, msg);
}
//
void errmsg2(const char* msg, const char* msg2) {
	char* format = "[-] KAFL ReFS: %s: %s\n";
	fprintf(stderr, format, msg, msg2);
}

void snerrmsg2(const char* msg, const char* msg2, char* buff, size_t size) {
	char* format = "[-] KAFL ReFS: %s: %s\n";
	snprintf(buff, size, format, msg, msg2);
}
