#ifndef KAFL_REFS_PRPC_H
#define KAFL_REFS_PRPC_H


/* kAFC ReFS payload remote procedure calls
 */

#include "../kafl_user.h"
#include <stddef.h>
#include <stdbool.h>

#ifdef __MINGW64__
	#include <windows.h>
#else
	// define neeeded types and structs
	typedef uint32_t DWORD;
	typedef void* LPVOID;
	typedef _Bool BOOL;
	typedef struct _SECURITY_ATTRIBUTES{
		DWORD nLength;
		LPVOID lpSecurityDecriptor;
		BOOL bInheritHandle;
	} SECURITY_ATTRIBUTES, *PSECURITY_ATTRIBUTES, *LPSECURITY_ATTRIBUTES;
#endif

//////  CALLS  //////

// implementation in refs_prpc_caller:
//	- creates kAFL_payload and sends
//	- return value: error code (Type: REFSCALL_RETVAL)
// implementation in corresponding refs_prpc_callee:
//	- conducts actual call
//	- do not call them directly!
//		- actually, this here is the wrong scope for acceptors
//		- They are just here also to assure identical function signatures
//		- invoked by accept_refscall(ReFSCall*)

int createDirectory(char* dirpath, SECURITY_ATTRIBUTES* securityAttributes);
int createFile(char* filename, uint32_t dwFlagsAndAttributes, SECURITY_ATTRIBUTES* securityAttributes); // many parameters left out for simplicity
int createSymLink(char* linkname, char* filename, uint32_t isdir);
int copyFile(char* src, char* dst);
int deleteFile(char* filename);
int getCompressedFileSize(char* filename);
int getFileAttributes(char* filename);
int getFileSize(char* filename);
int getFileTime(char* filename);
int getFileType(char* filename);
int moveFile(char* src, char* dst);
int lockFile(char* filename, uint32_t offset, uint32_t numbytes);
int removeDirectory(char* filename);
int setEndOfFile(char* filename, uint32_t offset);
int setFileAttributes(char* filename, uint32_t attributes);
// for windows: FILETIME* needed
int setFileTime(char* filename, void* creat, void* access, void* write);
int setFileValidData(char* filename, int64_t length);
int unlockFile(char* filename, uint32_t offset, uint32_t numbytes);
int readFile(char* filename);
int listDirectory(char* dirpath);
int copyDirectory(char* src, char* dst);
int createJunction(char* junctionPath, char* targetPath);
int copyJunction(char* srcJunction, char* dstJunction);

////////////////////////////////////////////////////////////////////////////////

//////  STRUCTURES  //////
// and iteration helpers

#ifdef __GNUC__
	#define PACKED __attribute__((packed))
#else
	#warning "Packed struct not implemented by kafl_refs_prpc.h for this compiler."
	#define PACKED
#endif
//
typedef struct PACKED {
	uint32_t memsize; // size of memory stored in mem
	uint8_t isnullptr;
	uint8_t mem[];
} ReFSCallParam;
//
typedef struct PACKED {
	int32_t id;
	int32_t params_size; // total size of params
	ReFSCallParam params[];
} ReFSCall;

// get next call
// @param call is current call
// @return successor of call, or NULL on list end
ReFSCall* getNextCall(ReFSCall* call);

// get first parameter
// @param call
// @return first parameter in call if present, else NULL
ReFSCallParam* getFirstParam(ReFSCall* call);

// get next parameter
// @param current parameter
// @return successor of param, or NULL on parameterlist end
ReFSCallParam* getNextParam(ReFSCallParam* param);

// @return true on call being a pseudocall created by setupListEndCall
int callIsListEnd(ReFSCall* call);
// @return true on param being a pseudoparam created by setupListEndParam
int paramIsListEnd(ReFSCallParam* param);

////////////////////////////////////////////////////////////////////////////////

enum REFSCALLID {
	LISTENDCALL=0,
	CREATEDIRECTORY=1,
	CREATEFILE=2,
	CREATENAMEDPIPE=3, // (? do pipes interact with refs ?)
	CREATESYMBOLICLINK=4,
	COPYFILE=5,
	DELETEFILE=6,
	GETCOMPRESSEDFILESIZE=7,
	GETFILEATTRIBUTES=8,
	GETFILESIZE=9,
	GETFILETIME=10,
	GETFILETYPE=11,
	MOVEFILE=12,
	LOCKFILE=13,
	REMOVEDIRECTORY=14,
	SETENDOFFILE=15,
	SETFILEATTRIBUTES=16,
	SETFILEINFORMATIONBYHANDLE=17,
	SETFILETIME=18,
	SETFILEVALIDDATA=19, // security warning on doc
	SETNAMEDSECURITYINFO=20,
	UNLOCKFILE=21,
	READFILE=22,
	LISTDIRECTORY=23,
	COPYDIRECTORY=24,
	CREATEJUNCTION=25,
	COPYJUNCTION=26,
};

// return values for refscall functions
enum REFSCALL_RETVAL {
	REFSCALL_SUCCESS=0,
	REFSCALL_GENERIC_ERROR=1,
	REFSCALL_HANDLE_OPEN_FAIL=2,
	REFSCALL_HANDLE_CLOSE_FAIL=3,
};

////////////////////////////////////////////////////////////////////////////////

//////  helpers for formatted output  //////

void dbgmsg(const char* msg);
void errmsg(const char* msg);
void errmsg2(const char* msg, const char* msg2);
void snerrmsg2(const char* msg, const char* msg2, char* buff, size_t size);

#endif

