#include "refs_prpc_callee.h"

#include <stdio.h>

// helper to get param #N of call
static void* pn(int n, ReFSCall* call) {
	ReFSCallParam* param = call->params;
	for (int i=0; i<n; ++i) {
		if (paramIsListEnd(param)) {
			// error, param #N does not exist
			errmsg("pn: number of parameters exceeded.");
			return NULL;
		}
		param = (ReFSCallParam*)(param->mem+param->memsize);
	}
	if (param->isnullptr)
		return NULL;
	return param->mem;
}
static int32_t pndw(int n, ReFSCall* call) {
	return *(int32_t*)pn(n, call);
}
static int64_t pnll(int n, ReFSCall* call) {
	return *(int64_t*)pn(n, call);
}

static int accept_refscall(ReFSCall* call)
{
	int retval = REFSCALL_SUCCESS;

	switch (call->id) {
	case LISTENDCALL:
		dbgmsg("listEndCall");
		retval = REFSCALL_GENERIC_ERROR;
		break;
	case CREATEDIRECTORY:
		dbgmsg("createDirectory");
		retval = createDirectory(pn(0,call), pn(1,call));
		break;
	case CREATEFILE:
		dbgmsg("createFile");
		retval = createFile(pn(0,call), pndw(1,call), pn(2,call));
		break;
	//case CREATENAMEDPIPE:
	//	break;
	case CREATESYMBOLICLINK:
		dbgmsg("createSymbolicLink");
		retval = createSymLink(pn(0,call),pn(1,call),pndw(2,call));
		break;
	case COPYFILE:
		dbgmsg("copyFile");
		retval = copyFile(pn(0,call),pn(1,call));
		break;
	case DELETEFILE:
		dbgmsg("deleteFile");
		retval = deleteFile(pn(0,call));
		break;
	case GETCOMPRESSEDFILESIZE:
		dbgmsg("getCompressedFileSize");
		retval = getCompressedFileSize(pn(0,call));
		break;
	case GETFILEATTRIBUTES:
		dbgmsg("getFileAttributes");
		retval = getFileAttributes(pn(0,call));
		break;
	case GETFILESIZE:
		dbgmsg("getFileSize");
		retval = getFileSize(pn(0,call));
		break;
	case GETFILETIME:
		dbgmsg("getFileTime");
		retval = getFileTime(pn(0,call));
		break;
	case GETFILETYPE:
		dbgmsg("getFileType");
		retval = REFSCALL_GENERIC_ERROR;
		break;
		retval = getFileType(pn(0,call));
		break;
	case MOVEFILE:
		dbgmsg("moveFile");
		retval = moveFile(pn(0,call),pn(1, call));
		break;
	case LOCKFILE:
		dbgmsg("lockFile");
		retval = lockFile(pn(0,call),pndw(1,call),pndw(2,call));
		break;
	case REMOVEDIRECTORY:
		dbgmsg("removeDirectory");
		retval = removeDirectory(pn(0,call));
		break;
	case SETENDOFFILE:
		dbgmsg("setEndOfFile");
		retval = setEndOfFile(pn(0,call),pndw(1,call));
		break;
	case SETFILEATTRIBUTES:
		dbgmsg("setFileAttributes");
		retval = setFileAttributes(pn(0,call),pndw(1,call));
		break;
	//case SETFILEINFORMATIONBYHANDLE:
	//	break;
	case SETFILETIME:
		dbgmsg("setFileTime");
		retval = setFileTime(pn(0,call),pn(1,call),pn(2,call),pn(3,call));
		break;
	case SETFILEVALIDDATA:
		dbgmsg("setFileValidData");
		retval = setFileValidData(pn(0,call),pnll(1,call));
		break;
	//case SETNAMEDSECURITYINFO:
	//	break;
	case UNLOCKFILE:
		dbgmsg("unlockFile");
		retval = unlockFile(pn(0,call),pndw(1,call),pndw(2,call));
		break;
	case READFILE:
		dbgmsg("readFile");
		retval = readFile(pn(0,call));
		break;
	case LISTDIRECTORY:
		dbgmsg("listDirectory");
		retval = listDirectory(pn(0,call));
		break;
	case COPYDIRECTORY:
		dbgmsg("copyDirectory");
		retval = copyDirectory(pn(0,call), pn(1,call));
		break;
	case CREATEJUNCTION:
		dbgmsg("createJunction");
		retval = createJunction(pn(0,call), pn(1,call));
		break;
	case COPYJUNCTION:
		dbgmsg("copyJunction");
		retval = copyJunction(pn(0,call), pn(1,call));
		break;
	default:
		{ // block needed for char[] declaration
			char emsg[64];
			snprintf(emsg, sizeof(emsg), "%s %d", "unknown ReFSCallId", call->id);
			errmsg(emsg);
		}
		retval = REFSCALL_GENERIC_ERROR;
		break;
	}
	return retval;
}

int accept_payload(ReFSCall* call)
{
	while (!callIsListEnd(call)) {
		int retval = accept_refscall(call);
		if (retval != REFSCALL_SUCCESS) {
			errmsg("accept_refscall");
			return 1;
		}
		call = (ReFSCall*)(((uint8_t*)call->params)+call->params_size);
	}
	return 0;
}

int accept_payload_debug(ReFSCall* call, void (*logfunction)(const char* msg))
{
	while (!callIsListEnd(call)) {
		int retval = accept_refscall(call);
		if (retval != REFSCALL_SUCCESS) {
			(*logfunction)("accept_refscall");
			if (retval == REFSCALL_GENERIC_ERROR) return 1;
		}
		call = (ReFSCall*)(((uint8_t*)call->params)+call->params_size);
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
