#include "refs_prpc_caller.h"

#include <stdio.h>
#include <string.h>

static kAFL_payload payloadbuf = {sizeof(int32_t)};
// keep state reminder of where to continue
// --> absolutely *not* thread-safe
// keep in struct and drag as parameter for better scope in case of need
static kAFL_payload* currPayload = &payloadbuf;
static ReFSCall* currCall = (ReFSCall*)&payloadbuf.data;
static ReFSCallParam* currParam;
//
ReFSCall* getPayload()
{
	return (ReFSCall*)currPayload->data;
}

int32_t getPayloadSize()
{
	return currPayload->size;
}

void setpayload(kAFL_payload* payload)
{
	currPayload = payload;
	currCall = (ReFSCall*)payload->data;
	setupListEndCall(currCall);
}
//
void reset()
{
	setpayload(&payloadbuf);
}


static int addParam(const void* src, size_t size);
////////////////////////////////////////////////////////////////////////////////

// setup next call in payload
ReFSCall* nextCall()
{
	addParam(NULL, 0); // terminate parameter list
	ReFSCall* nc = (ReFSCall*)(((uint8_t*)currCall->params)+currCall->params_size);
	// check for enough mem left in payload buffer
	void* payloadend = (void*)(currPayload+1);
	if ((void*)(nc+1) > payloadend) {
		return NULL;
	}
	currCall = nc;
	setupListEndCall(currCall); // call ist listend until instructed otherwise later
	currPayload->size += sizeof(ReFSCall);
	currParam = NULL;
	return currCall;
}


void setupListEndCall(ReFSCall* callbuf)
{
	callbuf->id = LISTENDCALL;
	callbuf->params_size = 0;
}
//
void setupListEndParam(ReFSCallParam* parambuf)
{
	parambuf->memsize = 0;
}

////////////////////////////////////////////////////////////////////////////////

// add parameter to call (in order)
static int addParam(const void* src, size_t size)
{
	// precomputed for capacity checks
	void* payloadend = (void*)(currPayload+1);
	//
	if (!currParam) {
		// setup param list
		ReFSCallParam* param = currCall->params;
		if ((void*)(((uint8_t*)param)+sizeof(ReFSCallParam)) > payloadend) {
			fprintf(stderr, "addParam: payload size exceeded");
			return 1;
		}
		setupListEndParam(param);
		currParam = param;
	}
	// check for NULL-pointer
	if (!src) {
		currParam->isnullptr = 1;
	} else {
		currParam->isnullptr = 0;
	}
	// check for enough memory left
	if ((void*)(currParam->mem+size) > payloadend) {
		fprintf(stderr, "addParam: payload size exceeded #2");
		return 1;
	}
	// copy value to buffer
	void* dst = &currParam->mem;
	if (size && src)
		memcpy(dst, src, size);
	currParam->memsize = size;
	currCall->params_size += sizeof(ReFSCallParam)+size;
	currPayload->size += sizeof(ReFSCallParam)+size;
	// end param list
	currParam = (ReFSCallParam*)(currParam->mem+currParam->memsize);
	setupListEndParam(currParam);
	return 0;
}

static int addStrParam(char* cp)
{
	if (addParam(cp, strlen(cp)+1))
		return 1;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

ReFSCall* prepCallBuf(int id)
{
	currCall->id = id;
	currCall->params_size = 0;
	return currCall;
}

static int call_cp(int id, char* cp)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}

static int call_cpsa(int id, char* cp, void* sa)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addParam(sa, sizeof(SECURITY_ATTRIBUTES)))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}

static int call_cpcp(int id, char* cp, char* cp2)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addStrParam(cp2))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}
static int call_cpcpdw(int id, char* cp, char* cp2, uint32_t dw)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addStrParam(cp2))
		return 1;
	if (addParam(&dw, sizeof(dw)))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}
static int call_cpdw(int id, char* cp, uint32_t dw)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addParam(&dw, sizeof(dw)))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}
static int call_cpdwsa(int id, char* cp, uint32_t dw, void* sa)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addParam(&dw, sizeof(dw)))
		return 1;
	if (addParam(sa, sizeof(SECURITY_ATTRIBUTES)))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}
static int call_cpdwdw(int id, char* cp, uint32_t dw, uint32_t dw2)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addParam(&dw, sizeof(dw)))
		return 1;
	if (addParam(&dw2, sizeof(dw2)))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}
#ifdef __MINGW64__
	// need to provide windows filetime structure via void*
	#define FILETIMESIZE sizeof(FILETIME)
#else
	// UNIX timestamps I suppose (time_t is assumed to be of pointer size)
	#define FILETIMESIZE sizeof(void*)
#endif
static int call_cpftftft(int id, char* cp, void* ft, void* ft2, void* ft3)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addParam(ft, FILETIMESIZE))
		return 1;
	if (addParam(ft2, FILETIMESIZE))
		return 1;
	if (addParam(ft3, FILETIMESIZE))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}
static int call_cpll(int id, char* cp, int64_t ll)
{
	prepCallBuf(id);
	if (addStrParam(cp))
		return 1;
	if (addParam(&ll, sizeof(ll)))
		return 1;
	if (nextCall())
		return 1;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int createDirectory(char* dirpath, SECURITY_ATTRIBUTES* securityAttributes)
{ return call_cpsa(CREATEDIRECTORY, dirpath, securityAttributes); }
//
int createFile(char* filename, uint32_t dwFlagsAndAttributes, SECURITY_ATTRIBUTES* securityAttributes)
{ return call_cpdwsa(CREATEFILE, filename, dwFlagsAndAttributes, securityAttributes); }
//
int createSymLink(char* linkname, char* filename, uint32_t dwFlags)
{ return call_cpcpdw(CREATESYMBOLICLINK, linkname, filename, dwFlags); }
//
int copyFile(char* src, char* dst)
{ return call_cpcp(COPYFILE, src, dst); }
//
int deleteFile(char* filename)
{ return call_cp(DELETEFILE, filename); }
//
int getCompressedFileSize(char* filename)
{ return call_cp(GETCOMPRESSEDFILESIZE, filename); }
//
int getFileAttributes(char* filename)
{ return call_cp(GETFILEATTRIBUTES, filename); }
//
int getFileSize(char* filename)
{ return call_cp(GETFILESIZE, filename); }
//
int getFileTime(char* filename)
{ return call_cp(GETFILETIME, filename); }
//
int getFileType(char* filename)
{ return call_cp(GETFILETYPE, filename); }
//
int moveFile(char* src, char* dst)
{ return call_cpcp(MOVEFILE, src, dst); }
//
int lockFile(char* filename, uint32_t offset, uint32_t numbytes)
{ return call_cpdwdw(LOCKFILE, filename, offset, numbytes); }
//
int removeDirectory(char* filename)
{ return call_cp(REMOVEDIRECTORY, filename); }
//
int setEndOfFile(char* filename, uint32_t offset)
{ return call_cpdw(SETENDOFFILE, filename, offset); }
//
int setFileAttributes(char* filename, uint32_t attributes)
{ return call_cpdw(SETFILEATTRIBUTES, filename, attributes); }
//
int setFileTime(char* filename, void* creat, void* access, void* write)
{ return call_cpftftft(SETFILETIME, filename, creat, access, write); }
//
int setFileValidData(char* filename, int64_t length)
{ return call_cpll(SETFILEVALIDDATA, filename, length); }
//
int unlockFile(char* filename, uint32_t offset, uint32_t numbytes)
{ return call_cpdwdw(UNLOCKFILE, filename, offset, numbytes); }
//
int readFile(char* filename)
{ return call_cp(READFILE, filename); }
//
int listDirectory(char* dirpath)
{ return call_cp(LISTDIRECTORY, dirpath); }
//
int copyDirectory(char* src, char* dst)
{ return call_cpcp(COPYDIRECTORY, src, dst); }
//
int createJunction(char* junctionPath, char* targetPath)
{ return call_cpcp(CREATEJUNCTION, junctionPath, targetPath); }
//
int copyJunction(char* srcJunction, char* dstJunction)
{ return call_cpcp(COPYJUNCTION, srcJunction, dstJunction); }

