/*

Copyright (C) 2017 Robert Gawlik

This file is part of kAFL Fuzzer (kAFL).

QEMU-PT is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

QEMU-PT is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with QEMU-PT.  If not, see <http://www.gnu.org/licenses/>.

*/

// Windows 10 setup
#define WINVER 0x0a00
#define _WIN32_WINNT 0x0a00
#include <windows.h>

#include <stdio.h>
#include "../../kafl_user.h"
#include "refs_prpc_callee_win.h"

#include "../../refs_expander/expander.h"

#include "../paths.h"

static char* info_buffer;

static void flushQemuDebug(){
	kAFL_hypercall(HYPERCALL_KAFL_DEBUG_PRINT, (UINT64)info_buffer);
}

void printQemu(char* text) {
	sprintf(info_buffer, text);
	flushQemuDebug();
}

static void initQemuDebugBuffer() {
    info_buffer = (char*)VirtualAlloc(0, DEBUG_BUFFER_SIZE, MEM_COMMIT, PAGE_READWRITE);
    memset(info_buffer, 0xff, DEBUG_BUFFER_SIZE);
    memset(info_buffer, 0x00, DEBUG_BUFFER_SIZE);
}

static void do_snwinerrmsg(const char* msg){
	snwinerrmsg(msg, info_buffer, DEBUG_BUFFER_SIZE);
	flushQemuDebug();
}

static HANDLE childvhd;
static int prepare_child_vhd(){
	// derive child VHD from parent
	VIRTUAL_STORAGE_TYPE type;
	type.DeviceId = VIRTUAL_STORAGE_TYPE_DEVICE_VHD;
	type.VendorId = VIRTUAL_STORAGE_TYPE_VENDOR_MICROSOFT;
	CREATE_VIRTUAL_DISK_PARAMETERS createparams;
	createparams.Version = CREATE_VIRTUAL_DISK_VERSION_1;
	GUID guid = {0}; // create new id
	createparams.Version1.UniqueId = guid;
	createparams.Version1.MaximumSize = 0; // zero for parent path specified
	createparams.Version1.BlockSizeInBytes = 0x200000;
	createparams.Version1.SectorSizeInBytes = 512;
	createparams.Version1.ParentPath = BASE_VHD_PATH_L; // PARENT_VHD_PATH fails (not found)
	createparams.Version1.SourcePath = NULL;
	DWORD createresult = CreateVirtualDisk(
		&type,
		CHILD_VHD_PATH,
		VIRTUAL_DISK_ACCESS_ALL,
		NULL,
		CREATE_VIRTUAL_DISK_FLAG_NONE,
		0,
		&createparams,
		NULL,
		&childvhd);
	if (createresult != ERROR_SUCCESS) {
		do_snwinerrmsg("[-] ReFS Fuzz child VHD creation failed");
		return 1;
	}
	// attach child VHD, includes mount
	ATTACH_VIRTUAL_DISK_PARAMETERS attachparams;
	attachparams.Version = ATTACH_VIRTUAL_DISK_VERSION_1;
	DWORD attachresult = AttachVirtualDisk(
		childvhd,
		NULL,
		ATTACH_VIRTUAL_DISK_FLAG_NONE, //|ATTACH_VIRTUAL_DISK_FLAG_NO_DRIVE_LETTER
		0,
		&attachparams,
		NULL);
	if (attachresult != ERROR_SUCCESS) {
		printQemu("[-] ReFS Fuzz: child VHD attach failed");
		return 1;
	}
	return 0;
}
static int delete_child_vhd(){
	int retval = 0;

	// detach vhd before closing handle
	DWORD detachresult = DetachVirtualDisk(
		childvhd,
		DETACH_VIRTUAL_DISK_FLAG_NONE,
		0);
	if (detachresult != ERROR_SUCCESS) {
		printQemu("[-] ReFS Fuzz: child VHD detach failed");
		retval = 1;
	}
	// close handle
	if (!CloseHandle(childvhd)) {
		printQemu("[-] ReFS Fuzz: child VHD handle close failed");
		retval = 1;
	}
	// delete file
	if (!DeleteFileW(CHILD_VHD_PATH)) {
		printQemu("[-] ReFS Fuzz: child VHD deletion failed");
		retval = 1;
	}
	return retval;
}

int main(int argc, char** argv)
{
	initQemuDebugBuffer();
	printQemu("[+] ReFS Fuzz: start");

	kAFL_payload* payload_buffer = (kAFL_payload*)VirtualAlloc(0, PAYLOAD_SIZE, MEM_COMMIT, PAGE_READWRITE);
	memset(payload_buffer, 0xff, PAYLOAD_SIZE); // assure memory is actually allocated

	kAFL_payload2* payload2_buffer = (kAFL_payload2*)VirtualAlloc(0, PAYLOAD2_SIZE, MEM_COMMIT, PAGE_READWRITE);
	memset(payload2_buffer, 0xff, PAYLOAD2_SIZE); // assure memory is actually allocated

	init_translation_table(IMAGE_TRANSLATION_TABLE_PATH);

	/* this hypercall submits the current CR3 value */
	kAFL_hypercall(HYPERCALL_KAFL_SUBMIT_CR3, 0);

	/* submit the guest virtual address of the payload buffer */
	kAFL_hypercall(HYPERCALL_KAFL_GET_PAYLOAD, (UINT64)payload_buffer);

	kAFL_hypercall(HYPERCALL_KAFL_GET_PAYLOAD2, (UINT64)payload2_buffer);

	while(1){

		/* request new payload (*blocking*) */
		printQemu("[+] ReFS Fuzz: request next payload");

		kAFL_hypercall(HYPERCALL_KAFL_NEXT_PAYLOAD, 0);
		kAFL_hypercall(HYPERCALL_KAFL_ACQUIRE, 0);

		printQemu("[+] ReFS Fuzz: expand payload");

		write_payload_to_file(payload2_buffer, BASE_VHD_PATH);


		/* kernel fuzzing */
		printQemu("[+] ReFS Fuzz: create overlay vhd and mount new FS");
		if (prepare_child_vhd()) {
			kAFL_hypercall(HYPERCALL_KAFL_RELEASE, 0);
			continue;
		}

		printQemu("[+] ReFS Fuzz: access FS via payload");
		int retval = accept_payload_debug((ReFSCall*)payload_buffer->data, &do_snwinerrmsg);
		if (retval) {
			printQemu("[-] ReFS Fuzz: accept_refscall failed");
		} else {
			printQemu("[+] ReFS Fuzz: accept_refscall success");
		}
		//unmount(guid);
		printQemu("[+] ReFS Fuzz: unmount and delete overlay vhd");
		delete_child_vhd();

		/* inform fuzzer about finished fuzzing iteration */
		kAFL_hypercall(HYPERCALL_KAFL_RELEASE, 0);
	}

	return 0;
}

